#ifndef __CMDREPLYFILE_H__
#define __CMDREPLYFILE_H__

#include "CmdReply.h"
#include "CmdClientTypes.h"

namespace Aspera {

namespace Command {

    class Buffer;

    /*! \brief
    *  Class to parse and hold the information of a single file
    */
    class File {
        private:
            const static char FileName      = 0x01;
            const static char FileSize      = 0x02;
            const static char FileMode      = 0x03;
            const static char FileSMode     = 0x04;
            const static char FileUid       = 0x05;
            const static char FileSUid      = 0x06;
            const static char FileGid       = 0x07;
            const static char FileSGid      = 0x08;
            const static char FileCTime     = 0x09;
            const static char FileSCTime    = 0x0A;
            const static char FileMTime     = 0x0B;
            const static char FileSMTime    = 0x0C;
            const static char FileATime     = 0x0D;
            const static char FileSATime    = 0x0E;
            const static char FileSymlink   = 0x0F;
            const static char FileError     = 0x10;
            const static char FileErrstr    = 0x11;
//             const static char DateFormat[]  = "yyyy-MM-dd kk:mm"; //Not sure how this works

            char       *sName;      // Filename in host character code
            int64_t        iSize;      // File size in bytes
            int         iMode;      // File mode as a int
            char       *sMode;      // File mode as a String
            int          iUid;      // UserId of the file as an int
            char        *sUid;      // UserId of the file as a String
            int          iGid;      // GroupId of the file as an int
            char        *sGid;      // GroupId of the file as a String
            int64_t       iCTime;      // Creation time as an int
            char       *sCTime;     // Creation time as a String
            int64_t       iMTime;      // Last modification time as an int
            char      *sMTime;      // Last modification time as a String
            int64_t       iATime;      // Last access time as an int
            char      *sATime;      // Last access time as a String
            char    *sSymlink;      // Symlink as a String
            int        iError;      // Errno on host
            char     *sErrstr;      // Error reason on host
            bool    directory;      // Directory
            bool  specialFile;      // Char or block device
            bool symbolicLink;      // Symbolic link
            bool      canRead;      // Has write permissions
            bool     canWrite;      // Has read permissions
//            SimpleDateFormat dateformat = null;     // Date formatter
//          TODO:Fix the above member

            CommandClientImpl *cmd_client;

        public:
            File (CommandClientImpl *cc);
            File();
            ~File();
            void  copyIn(File* aFile);
            int   processFileBuffer(Buffer *rawBuffer);
            void  printReply();
            File *nextFile;
            char* getName();
            int64_t getSize();
            char* getMode();
            char* getUid();
            char* getGid();
            char* getCTime();
            char* getMTime();
            char* getATime();
            char* getSymlink();
            int   getErrnum();
            char* getErrStr();
            bool  isDirectory();
            bool  isSpecialFile();
            bool  isSymbolicLink();
            bool  isCanRead();
            bool  isCanWrite();
    };

    /*! \brief
    *  Class to parse and hold the file list returned in response to an as_ls command
    */
    class CmdReplyFile : public CmdReply
    {
        private:
            int fileCount;
            File *fileListHead;
            File *fileListTail;

        public:
            CmdReplyFile(CommandClientImpl *cc);
            ~CmdReplyFile();
            int processMsgBuffer(Buffer *rawBuffer);
            void printReply();
            /*! \brief
            *  Returns a pointer to the array of File objects
            */
            File* getFileList();
            /*! \brief
            *  Returns the no. of files in the reply
            */
            int getFileCount(); //Does this need to be u_int32_t
    };

}
}
#endif
