#include "CommandClient.h"
#include "CmdReplyInfo.h"
#include "CmdReply.h"
#include "CmdReplyFile.h"
#include "CmdReplyMd5sum.h"
#include "CmdReplyMounts.h"
#include "CmdReplySize.h"
#include "CmdError.h"

int main()
{
    CommandClient *app = NULL;
    app = CommandClient::CreateInstance();
    CmdReplyInfo *rpi = NULL;
    CmdError cmderr;
    int filecount = 0;
    int rc = 0;

    /// CONNECT ///
	/// Connect to a remote machine running Aspera Command Server ///
    rpi = app->connect("demo.asperasoft.com","aspera","demoaspera",22,cmderr);

    /// REPLY INFO ///
    /// If the connection is successful print the server info. If not,  print the error message///
    if (rpi == NULL)
        cmderr.printError();
    else
        rpi->printReply();

    /// REPLY FILE "as_ls" ///
    /// Issue an ls command and print the file names to the stdout ///

    CmdReplyFile *rpf = NULL;
    rpf = (CmdReplyFile*) app->execLs("/",cmderr);
    if (rpf == NULL)
        cmderr.printError();
    else {
        File *filelist;
		//Retrieve the number of files returned by the remote server
        filecount = rpf->getFileCount(); 
        printf("\nNo. of files: %d",filecount);
		fflush(stdout);

        //Obtain a handle to the array of 'File' objects
        filelist = rpf->getFileList();

        //For each file returned print the filename
        for (int i = 0;i<filecount;i++,filelist++)
            printf("\nFile %d: %s",i+1,filelist->getName());
        fflush(stdout);
    }
    CmdReplyMd5sum *rpm  = NULL;

    /// REPLY MD5SUM "as_md5" ///
    /// Issue the md5sum command for a file named md5testfile ///

    rpm = (CmdReplyMd5sum*) app->execMd5sum("/aspera-test-dir-large/100MB",cmderr);
    if (rpm != NULL)
        printf("\n\nMd5sum: %s\n",rpm->getMd5sum());
    else
        cmderr.printError();

    /// REPLY MOUNTS "as_df" ///
    /// Issue a df command print the result to stdout ///

    CmdReplyMounts *rpd  = NULL;
    rpd = (CmdReplyMounts *) app->execDf(cmderr);

	//Print a summary of the reply
    if (rpd != NULL){
        rpd->printReply();
        printf("\n");
    }
    else
        cmderr.printError();

    /// REPLY SUCCESS "as_rm" ///
    /// Issue a command to remove a directory ///
    rc = 0;
    rc = app->execRm("/this-file-does-not-exist",cmderr);

        if (rc != 0){
            cmderr.printError();
        }
        else
            printf("\nRemove Successful");
        printf("\n");

    /// REPLY DISK USAGE "as_du" ///
    /// Issue a du command to find the disk usage of a file ///

    CmdReplySize *rps = NULL;
    rps = app->execDu("/aspera-test-dir-large",cmderr);
    if (rps == NULL)
        cmderr.printError();
    else
        rps->printReply();

    /// REPLY INFO "as_info" ///
    /// Issue an info command to fetch the server information ///
    delete rpi;
    rpi = (CmdReplyInfo *) app->execInfo(cmderr);

    if (rpi == NULL)
        cmderr.printError();
    else
        rpi->printReply();

	/// DISCONNECT ///
	/// Disconnect from the remote server ///
    app->disconnect();

	/// Free all the dynamically created objects ///
    delete rpi;
    delete rpf;
    delete rpm;
    delete rpd;
    delete app;

    printf("\nPress Enter to exit\n");
    getchar();

    return 0;
}
